@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <string.h>
 #include "gtkmain.h"
 #include "gtkprivate.h"
@@ -165,7 +165,7 @@ _gtk_xembed_send_message (GdkWindow        *recipient,
   gdk_error_trap_push ();
   XSendEvent (GDK_WINDOW_XDISPLAY(recipient),
 	      GDK_WINDOW_XWINDOW (recipient),
-	      False, NoEventMask, &xclient);
+	      False, NoEventMask, (XEvent *)&xclient);
   gdk_display_sync (display);
   gdk_error_trap_pop ();
 }
